# -*- coding: utf-8 -*-
"""
Created on Sun Jul 10 11:34:59 2022

@author: Noam Roth

compute the metric for a single cluster (neuron) in a recording
"""


from phylib.stats import correlograms
from types import SimpleNamespace
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import numpy as np
from scipy import stats
from scipy.optimize import curve_fit
import time



def slidingRP_all(spikeTimes, spikeClusters, **params):
    '''
    
    Compute the metric for each cluster in a recording
    
    Parameters
    ----------
    spikeTimes : numpy.ndarray
        array of spike times (ms)
    spikeClusters : numpy.ndarray
        array of spike cluster ids that corresponds to spikeTimes.
    params : dict
        params.binSizeCorr : bin size for ACG, usually set to 1/sampleRate (s)    TODO: set this up somewhere as same as refDur binsize? 
        params.sampleRate : sample rate of the recording (Hz)

    Returns
    -------
    rpMetrics: dict
        keys:
            maxConfidenceAt10Cont
            minContWith90Confidence 
            timeOfLowestCont
            nSpikesBelow2
            confMatrix (optional, if returnMatrix ==1)
        note: minContWith90Confidence, timeOfLowestCont will return np.nan 
        for neurons with too few spikes -- these neurons have "empty" 
        confidence and should be rejected. 
    cont: nd.array
        Vector of contamination values tested      
    rp: nd.array
        Vector of refractory period durations tested  

    '''

    if params and 'returnMatrix' in params:
        returnMatrix = params['returnMatrix'] 
    else:
        returnMatrix = False
    
    
    if params and 'verbose' in params:
        verbose = params['verbose']; 
    else:
        verbose = False
    
    
    cids = np.unique(spikeClusters)
    
    #initialize rpMetrics as dict
    rpMetrics = {}
    rpMetrics['cidx'] = []
    rpMetrics ['maxConfidenceAt10Cont'] = []
    rpMetrics['minContWith90Confidence'] = []
    rpMetrics['timeOfLowestCont'] = []
    rpMetrics['nSpikesBelow2'] = []
    
    if verbose:
        print("Computing metrics for %d clusters \n" % len(cids))
     

    for cidx in range(len(cids)):
        st = spikeTimes[spikeClusters==cids[cidx]] 
        
        [maxConfidenceAt10Cont, minContWith90Confidence, timeOfLowestCont,
            nSpikesBelow2, confMatrix, cont, rp, nACG,
            firingRate, secondsElapsed] = slidingRP(st, params)
    
        rpMetrics['cidx'].append(cids[cidx]) 
        rpMetrics['maxConfidenceAt10Cont'].append(maxConfidenceAt10Cont)
        rpMetrics['minContWith90Confidence'].append(minContWith90Confidence)
        rpMetrics['timeOfLowestCont'].append(timeOfLowestCont)
        rpMetrics['nSpikesBelow2'].append(nSpikesBelow2)
       
        
        if returnMatrix:
            if 'confMatrix' not in rpMetrics:
                rpMetrics['confMatrix'] = []
            rpMetrics['confMatrix'].append(confMatrix)
            
        
        if 'value' not in rpMetrics:
            rpMetrics['value'] = []
        if minContWith90Confidence<=10:
            rpMetrics['value'].append(1)
        else: 
            rpMetrics['value'].append(0)
            
        if verbose:
            if minContWith90Confidence<=10:
                pfstring = 'PASS'
            else: 
                pfstring = 'FAIL'
            print('  %d: %s max conf = %.2f%%, min cont = %.1f%%, time = %.2f ms, n below 2 ms = %d' % (cids[cidx], pfstring, maxConfidenceAt10Cont, minContWith90Confidence, timeOfLowestCont*1000, nSpikesBelow2))

    return rpMetrics


def slidingRP(spikeTimes, params = None):

    '''     
    Compute the metric for one cluster
    
    Parameters
    ----------
    spikeTimes : numpy.ndarray
        array of spike times (ms) for one cluster

    params : dict
        params.binSizeCorr : bin size for ACG, usually set to 1/sampleRate (s)    TODO: set this up somewhere as same as refDur binsize? 
        params.sampleRate : sample rate of the recording (Hz)

    Returns
    -------


    
    maxConfidenceAt10Cont:   Max confidence that you have <= 10% contamination
    minContWith90Confidence: Minimum contamination for which you have >=90% confidence
    timeOfLowestCont:        Time at which best score happens
    nSpikesBelow2:           Number of observed spikes that occur before 2 ms
    confMatrix:              Full confidence matrix of size nCont x nRP
    cont:Vector of contamination values tested 
    rp: Vector of refractory period durations tested  
    nACG: the autocorrelogram of the neuron
    firingRate: firing rate of the cluster, computed as the average acg value from 1-2 seconds
    '''

    if params is None:
        params = {}
        params['sampleRate'] = 30000
        params['binSizeCorr'] = 1 / params['sampleRate']
        params['returnMatrix'] = True
        params['verbose'] = True
        params['cidx'] = [0]


    seconds_start = time.time()
    [confMatrix, cont, rp, nACG, firingRate] = computeMatrix(spikeTimes, params)
    # matrix is [nCont x nRP]
    
    testTimes = rp>0.0005 # (in seconds) 
    #only test for refractory period durations greater than 0.5 ms
    
    maxConfidenceAt10Cont = max(confMatrix[cont==10,testTimes]) #TODO check behavior if no max
    
    
    indsConf90 = np.row_stack(np.where(confMatrix[:,testTimes]>90))
    ii = indsConf90[0] #row inds
    jj = indsConf90[1] #col inds
    

    try:
        minI = np.min(ii)
        idx = np.argmin(ii)
        minContWith90Confidence = cont[minI]
        minRP = np.argmax(confMatrix[minI,testTimes])


    except:    
        minContWith90Confidence = np.nan
    
        minRP = np.nan

    try:
        timeOfLowestCont = rp[minRP+np.where(testTimes)[0][0]+1]
    except: 
        timeOfLowestCont = np.nan
        
    
    nSpikesBelow2 = sum(nACG[0:np.where(rp>0.002)[0][0]+1])

    secondsElapsed = time.time()-seconds_start
    return maxConfidenceAt10Cont, minContWith90Confidence, timeOfLowestCont, nSpikesBelow2, confMatrix, cont, rp, nACG, firingRate, secondsElapsed
    
    
    
def computeMatrix(spikeTimes, params): 
    """
    Parameters
    ----------
    spikeTimes : numpy.ndarray
        array of spike times (ms)
    params : dict
        params.binSizeCorr : bin size for ACG, usually set to 1/sampleRate (s)    TODO: set this up somewhere as same as refDur binsize? 
        params.sampleRate : sample rate of the recording (Hz)

    Returns
    -------
    None.
    """
    
    
    cont = np.arange(0.5, 35, 0.5)  # vector of contamination values to test
    rpBinSize = 1 / 30000  
    rpEdges = np.arange(0, 10/1000, rpBinSize)  # in s
    rp = rpEdges + np.mean(np.diff(rpEdges)[0]) / 2 # vector of refractory period durations to test
    
    #compute firing rate and spike count
    n_spikes = spikeTimes.size
    #setup for acg
    clustersIds = [0]  # call the cluster id 0 (not used, but required input for correlograms)
    spikeClustersACG = np.zeros(n_spikes, dtype=np.int8)  # each spike time gets cluster id 0

    # compute an acg in 1s bins to compute the firing rate  
    nACG = correlograms(spikeTimes, spikeClustersACG, cluster_ids = clustersIds, bin_size = 1, sample_rate = params['sampleRate'], window_size=2,symmetrize=False)[0][0] #compute acg
    firingRate = nACG[1] / n_spikes
        
    nACG = correlograms(spikeTimes, spikeClustersACG, cluster_ids=clustersIds, bin_size=params['binSizeCorr'], sample_rate=params['sampleRate'], window_size=2, symmetrize=False)[0][0]  # compute acg

    # confMatrix = np.zeros((cont.size, rp.size)) * np.nan
    # for ir in np.arange(confMatrix.shape[1]):
    #     # compute observed violations
    #     obsViol = np.sum(nACG[0:ir + 1])  # TODO this is off slightly (half-bin) from matlab...
    #     for cidx in np.arange(confMatrix.shape[0]):
    #         confMatrix[cidx, ir] = 100 * computeViol(obsViol, firingRate, n_spikes, rp[ir] + rpBinSize / 2, cont[cidx] / 100)  # TODO FIX RP BIN

    confMatrix = 100 * computeViol(np.cumsum(nACG[0:rp.size])[np.newaxis, :], firingRate, n_spikes, rp[np.newaxis, :] + rpBinSize / 2, cont[:, np.newaxis] / 100)

    return confMatrix, cont, rp, nACG, firingRate


def computeViol(obsViol, firingRate, spikeCount, refDur, contaminationProp):
    '''
    

    Parameters
    ----------
    obsViol : int
        the number of spikes observed within the refractory period duration.
    firingRate : float
        firing rate of the cluster (estimated from ACG) in spks/s
    spikeCount : int
        total spike count of cluster
    refDur : float
        refractory period duration in seconds
    contaminationProp : float
        the allowed contamination (proportion) i.e. 0.1 for 10% contamination

    Returns
    -------
    confidenceScore : float
        how confident we are that the cluster is less than contaminationProp 
        contaminated, given the observed violations and refractory period for 
        this cluster, under a Poisson assumption

    '''

    contaminationRate = firingRate * contaminationProp 
    expectedViol = contaminationRate * refDur * 2 * spikeCount
    confidenceScore = 1 - stats.poisson.cdf(obsViol, expectedViol)
    
    return confidenceScore


def plotSlidingRP(spikeTimes, params = None):
    '''
    

    Parameters
    ----------
    spikeTimes : numpy.ndarray
        array of spike times (ms)
    params : dict
        params.binSizeCorr : bin size for ACG, usually set to 1/sampleRate (s)    TODO: set this up somewhere as same as refDur binsize? 
        params.sampleRate : sample rate of the recording (Hz)

    Returns
    -------
    None.

    '''
    if params is None: 
        clusterlabel = False

    [maxConfidenceAt10Cont, minContWith90Confidence, timeOfLowestCont,
        nSpikesBelow2, confMatrix, cont, rp, nACG, 
        firingRate,xx]  = slidingRP(spikeTimes, params)
    
    fig, axs = plt.subplots(nrows=1, ncols=3, figsize = (12,4))
    
    ax = axs[0]

    ax.bar(rp*1000, nACG[0:len(rp)], width = np.diff(rp)[0]*1000, color = 'k', edgecolor = (1, 0, 0, 0)) #TODO width??
    ax.set_xlim([0, 5]) 
    ax.set_xlabel('Time from spike (ms)')
    ax.set_ylabel('ACG count (spks)')
    if clusterlabel:
        t1 = ('Cluster #%d: FR=%.2f' %(params['cidx'][0], firingRate))
    ax.fill(np.array([0, 1, 1, 0])*0.5, np.array([0, 0, 1, 1])*ax.get_ylim()[1], 'k',alpha= 0.2)
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    
    
    ax = axs[1]
    c = ax.imshow(confMatrix, extent = [rp[0]*1000, rp[-1]*1000, cont[0], cont[-1]], aspect = 'auto', vmin = 0, vmax = 100, origin = 'lower')
    ax.set_xlim((0,5))
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['left'].set_visible(False)
    ax.spines['bottom'].set_visible(False)
    cbar = fig.colorbar(c, ax = ax, location = 'right')
    cbar.set_label('Confidence (%)')
    ax.invert_yaxis()
    ax.plot([rp[0]*1000, rp[-1]*1000], [10, 10], 'r', linewidth = 1)
    
    if ~np.isnan(timeOfLowestCont):
        ax.plot(timeOfLowestCont*1000*np.array([1, 1]), [cont[0], cont[-1]],'r', linewidth = 1)
    
        # compute the conf=90 contour
        #zeropad confMatrix 
        z = np.zeros((np.shape(confMatrix)[0]+1,np.shape(confMatrix)[1]))
        z[1:,:] = confMatrix
        
        ii = np.argmax(z>90, 0).astype(float)  
        ii[ii==0] = np.nan
        contContour = np.empty(np.shape(ii)); contContour[:] = np.nan
        contContour[~np.isnan(ii)] = cont[(ii[~np.isnan(ii)]-1).astype(int)]
        ax.plot(rp*1000, contContour, 'r', linewidth = 2)
    val = ax.get_ylim()[1]
    # ax.fill(np.array([0, 1, 1, 0])*0.5, np.array([0, 0, 1, 1])*ax.get_ylim()[1], 'k',alpha= 1) 
    ax.fill(np.array([0, 1, 1, 0])*0.5, np.array([0, 0, 1, 1])*ax.get_ylim()[1], 'k',alpha= 0.2)
    # ax.add_patch(patches.Rectangle((0,0), 0.5, val, fc = 'k') )
    ax.set_xlabel('Time from spike (ms)')
    ax.set_xlim([0, 5]) 
    ax.set_ylabel('Contamination (%)')
    # ax.set_ylim([max(cont),0]) #seems unnecessary?
    t2 = ('max conf = %.2f%%, min cont = %.1f%%, time = %.2f ms'% (maxConfidenceAt10Cont, minContWith90Confidence, timeOfLowestCont*1000))
    
    
    if minContWith90Confidence >= 10:
        axs[0].set_title(t1, color='r')                   
        axs[1].set_title(t2,color = 'r')
    elif nSpikesBelow2 == 0:
        axs[0].set_title(t1, color='b')                   
        axs[1].set_title(t2,color = 'b')
    else:
        axs[0].set_title(t1, color='g')                   
        axs[1].set_title(t2,color = 'g')
        
        
    
    ax = axs[2]
    ax.plot(rp*1000, np.squeeze(confMatrix[cont==10,:]), 'k', linewidth = 2.0)
    ax.set_xlabel('Time from spike (ms)')
    ax.set_ylabel('Confidence of \leq10% contamination (%)')
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    
    ax.plot([0, 5], [90, 90], 'r'); 
    ax.fill(np.array([0, 1, 1, 0])*0.5, np.array([0, 0, 1, 1])*ax.get_ylim()[1], 'k',alpha= 0.2)
    ax.set_xlim([0, 5]); 
    ax.set_ylim([0, 100]); 
    
    fig.tight_layout()


def fitSigmoidACG_All(spikeTimes, spikeClusters, brainRegions, spikeAmps, rp, params):

    cids = np.unique(spikeClusters)
    
    #initialize rpMetrics as dict
    rpFit = {}
    rpFit['cidx'] = []
    rpFit['rpEstimate']  = []
    rpFit['brainRegion'] = []
    rpFit['amp'] = []
    rpFit['fr'] = []

    for cidx in range(len(cids)):
        st = spikeTimes[spikeClusters==cids[cidx]] 
        brainRegion = brainRegions[cidx]
        fr = len(st)/(st[-1]-st[0])

        sa = spikeAmps[spikeClusters==cids[cidx]] 
        amp = np.nanmedian(sa)*1000000

        #estimate rp
        minFR = 1 #spks/s
        minAmp = 50 #uV
        
        try:
            estimatedRP, estimateIdx, xSigmoid, ySigmoid = fitSigmoidACG(st, rp, fr, amp, minFR, minAmp, params)

            rpFit['cidx'].append(cids[cidx]) 
            rpFit['rpEstimate'].append(estimatedRP)
            rpFit['brainRegion'].append(brainRegion)
            rpFit['fr'].append(fr)
            rpFit['amp'].append(amp)
            if verbose:

                print('Estimated RP for cluster %d is %.2f ms'%(cids[cidx],estimatedRP))
        except:
            continue

    return rpFit


def fitSigmoidACG(st, timeBins, fr, amp, minFR = 1, minAmp = 50, params = None):
    '''
    

    Parameters
    ----------
    acg : np.array
        heights of acg bins (probability of spike for each timebin)
    timeBins : np.array
        timeBins used when computing acg
    params : TYPE
        DESCRIPTION.

    Returns
    -------
    estimatedRP:
        
    x: 
        
    y: 

    '''
    if params and 'verbose' in params:
        verbose = params['verbose']; 
    else:
        verbose = False
        
    if params and 'sampleRate' in params:
        sampleRate = params['sampleRate']
    else:
        sampleRate = 30000
    if params and 'binSizeCorr' in params:
        binSizeCorr = params['binSizeCorr']
    else:
        binSizeCorr = 1/30000
    
    
    
    #setup for acg
    clustersIds = [0] # call the cluster id 0 (not used, but required input for correlograms)
    spikeClustersACG = np.zeros(len(st), dtype = 'int8') # each spike time gets cluster id 0 
    #compute acg
    acg = correlograms(st, spikeClustersACG, cluster_ids = clustersIds, bin_size = binSizeCorr, sample_rate = sampleRate, window_size=2,symmetrize=False)[0][0] #compute acg

    if len(acg) > len(timeBins):
        acg = acg[0:len(timeBins)]

    if 'numSpikesThresh' not in params.keys():
        numSpikesThresh = 20  # need at least this many total spikes to compute fit
    else:
        numSpikesThresh = params['numSpikesThresh']
    
    peakDistFromEndBin = 5 #todo: params!! 
        
    estimatedRP = np.nan #initialize as a nan for cases where it doesn't work
    if fr >= minFR and amp >= minAmp:    #(sum(acg)>numSpikesThresh):   
        #potential todo: insert a case here for if the acg is symmetric?
        # p0 = [np.mean(ydata),2,1,min(ydata)] #starting point for fit? 
        minSigmoid = np.mean(acg[timeBins<0.0005]) #first 0.5 ms of data #todo make this parameter
        peakIdx = np.argmax(acg)        
        peakVal = np.max(acg)
        
        #2ms around peak
        timeValuesMin = np.where(timeBins >= timeBins[peakIdx]-0.001)[0][0]
        timeValuesMax = np.where(timeBins <= timeBins[peakIdx]+0.001)[0][-1]
        maxSigmoid = np.mean(acg[timeValuesMin:(timeValuesMax+1)])
    
        #if the peak is well before the end of the acg, only fit data up to the peak + n bins
        if peakIdx < len(acg)-peakDistFromEndBin:
            acg = acg[0:peakIdx + peakDistFromEndBin]
            timeBins = timeBins[0:peakIdx + peakDistFromEndBin]
    
        #fit the sigmoid with max and min fixed
        try:
            popt, pcov = curve_fit(lambda x, x0, k: sigmoid(x, maxSigmoid, x0, k, minSigmoid ), timeBins, acg)      
            fitParams = [maxSigmoid, popt[0], popt[1], minSigmoid]
    
            xSigmoid = timeBins#np.linspace(0, timeBins[-1], timeBins[-]) #evenly spaced vector for plotting sigmoid
            ySigmoid = sigmoid(xSigmoid, *fitParams)
        
                #find RP            
            RPEstimateFromPercentageOfSlope = 0.10
            estimateIdx, _ = closest(ySigmoid, RPEstimateFromPercentageOfSlope*(maxSigmoid - minSigmoid) + minSigmoid) 
            estimatedRP = 1000* xSigmoid[estimateIdx] # in ms
    
        except:
            estimatedRP = np.nan
            estimateIdx = np.nan
            xSigmoid = np.nan
            ySigmoid = np.nan
    
                #print('fit error')

        

    else:
        #print('fit error')
        estimatedRP = np.nan
        estimateIdx = np.nan
        xSigmoid = np.nan
        ySigmoid = np.nan
        
    return estimatedRP, estimateIdx, xSigmoid, ySigmoid
    


def plotSigmoid(ax, acg, timeBins, ySigmoid, estimatedIdx, estimatedRP):
    if len(acg) > len(timeBins):
        acg = acg[0:len(timeBins)]
    ax.bar(timeBins*1000, acg, width = np.diff(timeBins*1000)[0],alpha = 0.5)
    ax.set_xlim(0,5)

    ax.plot(timeBins*1000, ySigmoid,'k')
    ax.plot(timeBins[estimatedIdx]*1000, ySigmoid[estimatedIdx],'rx')
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.set_title('Estimated RP:%.2f ms'%estimatedRP)
    ax.set_ylabel('Number of spikes')
    ax.set_xlabel('Time (s)')
    return acg
    
    
    
    
    
    #helper functions    
def find_nearest(array, value):
    array = np.asarray(array)
    subtracted = (array - value)
    valid_idx = np.where(subtracted >= 0)[0]
    if len(valid_idx)>0:

        out = valid_idx[subtracted[valid_idx].argmin()]

    else:
        out = np.nan
    
    return out


def sigmoid(x, L, x0, k, b): # L is max value; x0 is the midpoint x value; k is the steepness, b is the baseline shift
    y = L / (1 + np.exp(-k*(x-x0)))+b
    return y


def closest(lst, K):
      
     lst = np.asarray(lst)
     idx = (np.abs(lst - K)).argmin()
     return idx, lst[idx]


# -*- coding: utf-8 -*-
"""
Created on Wed Jul 13 10:58:20 2022

@author: Noam Roth

Script for running and testing slidingRP functions
"""

#imports

import numpy as np
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy.stats import gaussian_kde

import sys
sys.path.append(r'C:\Users\Steinmetz Lab User\int-brain-lab\paper-reproducible-ephys')
sys.path.append(r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\metrics\slidingRP')
sys.path.append(r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python')
#from reproducible_ephys_functions import query
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import mpl_scatter_density # adds projection='scatter_density'
from matplotlib.colors import LinearSegmentedColormap
from one.api import ONE
from slidingRP import *
import pickle
one = ONE()

#%% load data

params = {}
params['sampleRate'] = []
params['sampleRate'] = 30000
params['binSizeCorr'] = 1 / params['sampleRate']
# params = SimpleNamespace(**params) #convert to dot notation
params['returnMatrix'] = True
params['verbose'] = True

#%%
# mat = computeMatrix(spikeTimes, params)
#load sample data
datapath = r'E:\Hopkins_CortexLab'
nickname = 'Hopkins'
spikeTimes = np.load(datapath + '\\spike_times.npy').flatten() 
if nickname == 'Hopkins':
    #convert from samples to seconds
    spikeTimes = spikeTimes/params['sampleRate']

spikeClusters = np.load(datapath + '\\spike_clusters.npy')

#%%
#run slidingRP for one cluster
params['cidx'] = [20] #or 280
st = spikeTimes[spikeClusters == params['cidx'][0]]
print(len(st))
[maxConfidenceAt10Cont, minContWith90Confidence, timeOfLowestCont,
nSpikesBelow2, confMatrix, cont, rp, nACG,
firingRate,xx] = slidingRP(st, params)




 #%%
#run plotting code for one cluster
#for RE presentation: ran below plus plotSlidingRP with a temporary "bug"
params = {}
params['cidx'] = [150]
st = spikeTimes[spikeClusters == params['cidx'][0]]
plotSlidingRP(st, params)
from brainbox.singlecell import firing_rate, acorr

hist_win = 0.05
fr_win = 1
if(1):
    fig, ax = plt.subplots(nrows=1, ncols=1, figsize = (12,4))
    st = st[:-500]
    fr = firing_rate(st, hist_win = hist_win, fr_win=fr_win)
    x = np.arange(fr.size) * hist_win
    ax.plot(x, fr, color='k', linewidth=0.4)
    ax.set_title('Firing Rate')
    ax.set_xlabel('Time (s)')
    ax.set_ylabel('Rate (spks/s)')



#%%
#run slidingRP for the loaded recording

[rpMetrics, cont, rp, sc, frrd] = slidingRP_all(spikeTimes, spikeClusters, params = params)
#%%


fig,ax = plt.subplots(1,1)
ax.scatter(sc[0:10],frrd[0:10])
ax.set_xlabel('spikecount')
ax.set_ylabel('firingrate * recDur')
xlims = ax.get_xlim()
ax.plot( [0,xlims[1]],[0,xlims[1]] )

#%%
#look into neurons where spike count is greater than firing rate * recDur 
# these should be neurons that now pass (spike count) and previously failed  (fr * recDur)
candidateNeurs = np.where((sc-frrd)>0)[0]

oldVal = np.array([rpMetricsUpdated['oldMetricValue'][x] for x in candidateNeurs])
newVal = np.array([rpMetricsUpdated['value'][x] for x in candidateNeurs])

print(newVal-oldVal) # there is only one such neuron!

#%%
# try it the other way:
oldVal = np.array([rpMetricsUpdated['oldMetricValue'][x] for x in range(len(rpMetricsUpdated['oldMetricValue']))])
newVal = np.array([rpMetricsUpdated['value'][x] for x in range(len(rpMetricsUpdated['oldMetricValue']))])

idxFailPass = np.where(newVal-oldVal > 0 )[0]
fig,ax = plt.subplots(1,1)
ax.scatter(sc[idxFailPass], frrd[idxFailPass])
ax.set_xlabel('spikecount')
ax.set_ylabel('firingrate * recDur')
xlims = ax.get_xlim()
ax.plot( [0,xlims[1]],[0,xlims[1]] )


#%% script for testing sigmoid fits
estimatedRP, estimateIdx, xSigmoid, ySigmoid = fitSigmoidACG(nACG, rp, params)



#%%
fig,axs = plt.subplots(1,1,figsize = (5,3))
ax = axs
acg = nACG
timeBins = rp
if len(acg) > len(timeBins):
    acg = acg[0:len(timeBins)]
print(acg)
ax.bar(timeBins*1000, acg, width = np.diff(timeBins*1000)[0],alpha = 0.5)
ax.set_xlim(0,10)
x = timeBins*1000

ax.plot(x[0:len(ySigmoid)], ySigmoid,'k')
ax.plot(timeBins[estimateIdx]*1000, ySigmoid[estimateIdx],'rx')
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.set_title('Estimated RP: %.2f ms'%estimatedRP)
ax.set_ylabel('Number of spikes')
ax.set_xlabel('Time (ms)')



# acg = plotSigmoid(ax, nACG, xSigmoid, ySigmoid, estimateIdx, estimatedRP)
plt.savefig(r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\exampleACG.pdf', dpi=300, format='pdf')


#%%
brainRegions = np.empty(len(np.unique(spikeClusters)))
rpEstimates = fitSigmoidACG_All(spikeTimes, spikeClusters, brainRegions, rp, params)


# -*- coding: utf-8 -*-
"""
Created on Mon Jul 25 22:10:45 2022

@author: Noam Roth

fit estimate histograms

"""


import numpy as np
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy.stats import gaussian_kde


#%% IBL 

import sys
sys.path.append(r'C:\Users\Steinmetz Lab User\int-brain-lab\paper-reproducible-ephys')
sys.path.append(r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\metrics\slidingRP')
sys.path.append(r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python')
#from reproducible_ephys_functions import query
import ssl
ssl._create_default_https_context = ssl._create_unverified_context
import mpl_scatter_density # adds projection='scatter_density'
from matplotlib.colors import LinearSegmentedColormap
from one.api import ONE
from slidingRP import *
import pickle
one = ONE()


from phylib.stats import correlograms
from RP_plotting import *
import matplotlib as mpl
mpl.rcParams['pdf.fonttype'] = 42 #sets fonts as editable

#pull down RS data
from reproducible_ephys_functions import filter_recordings, labs, BRAIN_REGIONS, query, get_insertions
from reproducible_ephys_functions import combine_regions, BRAIN_REGIONS, get_insertions, save_data_path, save_dataset_info



from brainbox.io.one import SpikeSortingLoader
from ibllib.atlas import AllenAtlas
ba = AllenAtlas()

insertions = get_insertions(level=2, one=one, freeze='biorxiv_2022_05')
savefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\savedRSFits\\'
#set up params for slidingRP_all
params = {}
params['sampleRate'] = []
params['sampleRate'] = 30000
params['binSizeCorr'] = 1 / params['sampleRate']
params['returnMatrix'] = True
params['verbose'] = False


cont = np.arange(0.5, 35, 0.5)  # vector of contamination values to test
rpBinSize = 1 / 30000  
rpEdges = np.arange(0, 10/1000, rpBinSize) # in s  
rp = rpEdges + np.mean(np.diff(rpEdges)[0])/2 

all_recordings = []
for iIns, ins in enumerate(insertions):

    try:
        print(f'processing {iIns + 1}/{len(insertions)}')
        eid = ins['session']['id']
        probe = ins['probe_name']
        pid = ins['probe_insertion']
    
    
        # Load in spikesorting
        sl = SpikeSortingLoader(eid=eid, pname=probe, one=one, atlas=ba)
        spikes, clusters, channels = sl.load_spike_sorting()
        clusters = sl.merge_clusters(spikes, clusters, channels)
    
    
      
    
        clusters['rep_site_acronym'] = combine_regions(clusters['acronym'])
        # Find clusters that are in the repeated site brain regions
        cluster_idx = np.where(np.isin(clusters['rep_site_acronym'], BRAIN_REGIONS))[0]
        brainRegions  = [clusters['rep_site_acronym'][i] for i in cluster_idx]

        #save the 'old' (computed in pipeline) slidingRP metric value
        # oldMetricValue = [clusters['slidingRP_viol'][x] for x in cluster_idx]
        
        #now rerun code in steinmetzlabrepo
        print('Loading spike times and clusters...')
        cluInds = np.array(list(x for x in range(len(spikes.clusters)) if spikes.clusters[x] in cluster_idx)) #this takes forever??
        spikeTimes = spikes.times[cluInds]
        spikeClusters = spikes.clusters[cluInds]
        spikeAmps = spikes.amps[cluInds]
        
        print('Fitting RP Estimate...')
        
        # compute estimated RP for all neurons
        rpEstimates = fitSigmoidACG_All(spikeTimes, spikeClusters,brainRegions, spikeAmps, rp, params)
        
        with open(savefile + ins['session']['subject'] + '.pickle', 'wb') as handle:
            pickle.dump(rpEstimates, handle)
    
    except:
        print('error')
        
        
# Nick data

from one.api import ONE
import ssl
ssl._create_default_https_context = ssl._create_unverified_context

savefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\savedSteinmetzFits\\'

one = ONE(cache_dir='E:\Steinmetz2019\Steinmetz_et_al_2019_9974357\9974357') # The location of the unarchived data
sessions = one.search()

BRAIN_REGIONS = ['ACA', 'AUD','ILA' , 'MOp', 'MOs',  'OLF', 'ORB', 'ORBm',
                     'PIR', 'PL', 'RSP', 'SSp','SSs',  'VISa', 'VISam', 'VISl',
                     'VISp', 'VISpm', 'VISrl',
                 'CA', 'CA1', 'CA2', 'CA3','DG', 'POST', 'SUB'
                 'TH', 'CL', 'LD', 'LGd', 'LH', 'LP', 'MD', 'MG','PO', 'POL', 
                     'PT','RT','SPF','VAL', 'VPL', 'VPM', 
                 'SNr','APN', 'IC','MB','MRN', 'NB','PAG','RN','SCig', 'SCm', 
                     'SCs', 'SCsg']
rpBinSize = 1 / 30000  
rpEdges = np.arange(0, 10/1000, rpBinSize) # in s  
rp = rpEdges + np.mean(np.diff(rpEdges)[0])/2 # vector of refractory period durations to test 
params = {}

for e,eid in enumerate(sessions):
    
        print(f'processing {e + 1}/{len(sessions)}')
        
        print('Loading spike times and clusters...')
        # Load data
        try:
            spikes, clusters, channels =pickle.load(open("E:\Steinmetz2019\loaded_data\data_{eid}.p".format(eid=eid), "rb"))
        except FileNotFoundError:
            channels = one.load_object(eid,'channels')
            spikes = one.load_object(eid,'spikes')
            clusters = one.load_object(eid, 'clusters')
            pickle.dump((spikes, clusters, channels),(open("E:\Steinmetz2019\loaded_data\data_{eid}.p".format(eid=eid), "wb")))    
            

        #find all descendant acronyms of brain area of interest
        cluster_areas =[channels.brainLocation.allen_ontology[int(clusters.peakChannel[i])-1] for i in range(len(clusters.peakChannel))]
        cluster_idx = np.where(np.isin(cluster_areas, BRAIN_REGIONS))[0]
        brainRegions  = [cluster_areas[i] for i in cluster_idx]

        cluInds = np.array(list(x for x in range(len(spikes.clusters)) if spikes.clusters[x] in cluster_idx)) #this takes forever??
        spikeTimes = spikes.times[cluInds]
        spikeClusters = spikes.clusters[cluInds]
        spikeAmps = spikes.amps[cluInds]
        
        print('Fitting RP Estimate...')
        # compute estimated RP for all neurons
        rpEstimates = fitSigmoidACG_All(spikeTimes, spikeClusters,brainRegions, spikeAmps, rp, params)

        with open(savefile + eid + '.pickle', 'wb') as handle:
            pickle.dump(rpEstimates, handle)

#%% load allen data

#code to save all allen data in a dataframe is here: Desktop\ecephys_data_access-Copy1

#TODO: edit this filename and rest of this code to run on recently saved (all!) data
#make sure you know that spike times are in useconds i think? check that

#try to rerun for LP only and see what happens
#TODO plotting: make the histograms smooth like in Allen log firing rate plot 



import pickle
import pandas as pd


savefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\savedAllenFits\\'

rpBinSize = 1 / 30000  
rpEdges = np.arange(0, 10/1000, rpBinSize) # in s  
rp = rpEdges + np.mean(np.diff(rpEdges)[0])/2 # vector of refractory period durations to test 
params = {}

minFR = 1
minAmp = 50

#find session names (from the saved files)
from os import listdir
from os.path import isfile, join
mypath = 'E:\AllenBrainObservatory\saved_units'
onlyfiles = [f for f in listdir(mypath) if isfile(join(mypath, f))]
sessions = [i[11:20] for i in onlyfiles]

for j, session_id in enumerate(sessions):
    file_name = 'E:\AllenBrainObservatory\saved_units\Allen_units%s.pkl'%session_id
    print('loading data for session %d out of %d'%(j+1, len(sessions)))

    #load the dataframe for this session
    with open(file_name, 'rb') as f:
        units = pickle.load(f)


    #initialize rpEstimates dict
    rpEstimates = {}
    rpEstimates['cidx'] = []
    rpEstimates['rpEstimate']  = []
    rpEstimates['brainRegion'] = []
    rpEstimates['parentRegion'] = [] 
    rpEstimates['amp'] = []
    rpEstimates['fr'] = []
    
    for i, (unit_id, row) in enumerate(units.iterrows()):
        st = row.spike_times
        timeBins = rp
        fr = row.firing_rate
        amp = row.waveform_amplitude
        brainRegion = row.subregion
        parentRegion = row.region

        # print('Estimating RP Estimate...')
        # compute estimated RP for all neurons
        estimatedRP, estimateIdx, xSigmoid, ySigmoid = fitSigmoidACG(st, rp, fr, amp, minFR, minAmp, params)

        rpEstimates['cidx'].append(unit_id) 
        rpEstimates['rpEstimate'].append(estimatedRP)
        rpEstimates['brainRegion'].append(brainRegion)
        rpEstimates['parentRegion'].append(parentRegion)
        rpEstimates['fr'].append(fr)
        rpEstimates['amp'].append(amp)
        
        

    with open(savefile + session_id + '.pickle', 'wb') as handle:
        pickle.dump(rpEstimates, handle)


#%%
# file_name = savefile + session_id + '.pickle'
# with open(file_name, 'rb') as f:
#     rpEstimates = pickle.load(f)

#look into neuron 2 of session 2 -- huge amplitude but very short rp?? 




# -*- coding: utf-8 -*-
"""
Created on Wed Jul 13 14:40:23 2022

@author: Noam Roth

code to run simulations for slidingRefractory metric
"""
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import datetime

import time



def genST(rate, duration, params = None):
    '''
    

    Parameters
    ----------
    rate : float
        firing rate of simulated neuron (spks/s).
    duration: float
        length of recording of simulated neuron (s)
    params : dict, optional
        Todo: here include any parameters like bursty, drifty? The default is None.

    Returns
    -------
    st: np.array
        array of spike times in seconds. 

    '''
    # print('generating spike train...')
    mu = 1 / rate
    n = rate * duration
    isi = np.random.exponential(mu, int(np.ceil(n * 2))) #generate 
    while sum(isi) < duration:
        np.append(isi, np.random.exponential(mu))

    st = np.cumsum(isi)
    if len(np.where(st<duration)[0])>0:
        st = st[0:np.where(st < duration)[0][(-1)]]
    else:
        st =[]

    if params['checkFR']:
        print('plotting FR...')
        fig,ax = plt.subplots(1,1)
        histWin = 0.5 #seconds
        binnedFR = firing_rate(st,hist_win = histWin, fr_win =  10)
        ax.plot(np.arange(binnedFR.size) * histWin, binnedFR, 'k')
        ax.set_xlabel('Time (s)')
        ax.set_ylabel('FR (spks/s)')
        ax.set_title('FR for simulated neuron, mean %d spikes/second, duration %d seconds'%(rate, duration) )

    return st


def simulateContNeurons(params):

    
    #initialize matrices of percent neurons pass
    passPct = np.empty([len(params['recDurs']), len(params['RPs']),len(params['baseRates']), len(params['contRates'])])
    passPct[:] = np.nan
    passPct2MsNoSpikes = np.zeros([len(params['recDurs']), len(params['RPs']),len(params['baseRates']), len(params['contRates'])])
    passPct2MsNoSpikes[:] = np.nan
    
    # passPctOld = np.zeros([len(recDurScalarVec), len(rpvec),len(baseRates), len(contPct)])
    start_time = time. time()
    
    
    #set up plotting
    
    for j, recDurScalar in enumerate(params['recDurs']):
        recDur = recDurScalar*3600
        print('recording Duration %d'%recDur)   
        for i, rp in enumerate(params['RPs']):
            print('refractory period duration %.3f'%rp)
            thresh = params['threshold']
    
            bidx=0
            for baseRate in params['baseRates']:
                print('baseRate %.2f'%baseRate)
                cidx=0
                for c in params['contRates']:
                    contRate = baseRate*c
                    print('contRate %.2f'%c)

                    # rpidx=np.where(b<rp)[0][-1]+1 #???????
                    # mfunc =np.vectorize(genST)
                    #### stopped above here. 
    
    #in the old version, I used max acceptable: computed a max acceptable for this set of parameters
    # and then (in the loop below nSim) simulated a bunch of neurons and compared each to this same max acceptable
    
                    # m = mfunc(baseRate,b[1:-1],recDur,baseRate/10,thresh)
        
                    
                    # simRes = np.zeros([nSim,len(b[1:-1])])
                    # simResOld = np.zeros([nSim])
                    passVec = np.empty(params['nSim'])
                    passVec[:] = np.nan
                    passVec2MsNoSpikes = np.empty(params['nSim'])
                    passVec2MsNoSpikes[:] = np.nan
                    for n in range(params['nSim']):
                        st = genST(baseRate,recDur,params) #generate a spike train with the current base rate
                        isi = np.diff(np.insert(st,0,0)) 
                        # print(rp)
                        isi = np.delete(isi,np.where(isi<rp)[0]) #get rid of already contaminating spikes (does this make sense??? why are there already contaminating spikes)
                        st = np.cumsum(isi)
                        if c>0:
                            contST = genST(contRate,recDur,params)
                        else:
                            contST=[]
                        combST = np.sort(np.concatenate((st, contST))) # put spike times from both spike trains together (and sort chronologically)
                        
                        [maxConfidenceAt10Cont, minContWith90Confidence, timeOfLowestCont,
                             nSpikesBelow2, confMatrix, cont, rpVec, nACG,
                             firingRate, secondsElapsed] = slidingRP(combST, params)
                        
                        if minContWith90Confidence <=10:
                            passVec[n] = 1
                        else:
                            passVec[n] = 0
                        
                        passVec2MsNoSpikes[n] = passVec[n]
                        if nSpikesBelow2 ==0:
                            passVec2MsNoSpikes[n] = 1
                        # c0 = correlograms(combST,np.zeros(len(combST),dtype='int8'),cluster_ids=[0],bin_size=binSize,sample_rate=20000,window_size=.05,symmetrize=False)
                        # simRes[n,:] = np.cumsum(c0[0,0,1:(len(b)-1)])
                        # len(simRes)
                        # simResOld[n] = contamination_alt(np.asarray(combST))
                        # #pass_vec_old.append(int(ce<0.1))
                    passPct[j, i, bidx,cidx]=sum(passVec)/params['nSim']*100
                    passPct2MsNoSpikes[j, i, bidx,cidx]=sum(passVec2MsNoSpikes)/params['nSim']*100

                    # passPctOld[j, i, bidx,cidx]= sum([ce<0.1 for ce in simResOld])/nSim*100
                    cidx+=1
    
                bidx+=1
    
    current_time = time. time()
    elapsed_time = current_time - start_time
    print('Loop with %f iterations takes %f seconds' % (params['nSim'], elapsed_time))       
    return passPct, passPct2MsNoSpikes
         
    
def plotSimulations(pc,params, savefile):
 
    colors = matplotlib.cm.rainbow(np.linspace(0, 1, len(params['baseRates'])))
    fig,axs = plt.subplots(len(params['recDurs']),len(params['RPs']), figsize = (12,3*len(params['recDurs'])))

    for j, recDur in enumerate(params['recDurs']):
        for i, rp in enumerate(params['RPs']):
            
            if len(params['recDurs']) > 1 and len(params['RPs'])>1:
                ax = axs[j,i]
            else:
                ax = axs[(j+1)*i]
            #different base rates get different colors
            for b, baseRate in enumerate(params['baseRates']):
                ax.plot(params['contRates'], pc[j, i, b,:], '.-',color = colors[b], label = baseRate)
                ax.set_ylabel('Percent pass')
                ax.set_xlabel('Proportion contamination')
                ax.set_title('True RP %d ms'%(rp*1000))
        fig.text(0.425, 0.9-(.17*j), 'Recording duration: %d hours'%recDur)
   
    handles, labels = ax.get_legend_handles_labels()
    fig.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=0.5, hspace=1.2)

    fig.legend(handles, labels, loc='upper right')


    fig.savefig(savefile, dpi = 500)
    
    
    fig,axs = plt.subplots(len(params['contRates'][::2]),len(params['RPs']), figsize = (12*2,3*len(params['recDurs'])))
    for j, contRate in enumerate(params['contRates'][::2]):
        print(j)
        for i, rp in enumerate(params['RPs']):
            
            if len(params['recDurs']) > 1 and len(params['RPs'])>1:
                ax = axs[j,i]
            else:
                ax = axs[(j+1)*i]
            #different base rates get different colors
            for b, baseRate in enumerate(params['baseRates']):
                ax.plot(params['recDurs'], pc[:, i, b,j*2], '.-',color = colors[b], label = baseRate)
                ax.set_ylabel('Percent pass')
                ax.set_xlabel('recording Duration')
                ax.set_title('True RP %d ms'%(rp*1000))
        fig.text(0.65, 0.9-(.17*j), 'Proportion contamination: %.2f'%contRate)
    fig.subplots_adjust(left=0.5, bottom=None, right=None, top=None, wspace=0.5, hspace=1.2)

    fig.legend(handles, labels, loc='upper right')
    fig.show()
    
    
    
    fig,axs = plt.subplots(len(params['recDurs']), len(params['contRates'][::2]), figsize = (12*2,3*len(params['recDurs'])))
    for j, recDur in enumerate(params['recDurs']):
        print(j)
        for i, contRate in enumerate(params['contRates'][::2]):
            
            if len(params['recDurs']) > 1 and len(params['contRates'])>1:
                ax = axs[j,i]
            else:
                ax = axs[(j+1)*i]
            #different base rates get different colors
            for b, baseRate in enumerate(params['baseRates']):
                ax.plot(params['RPs'], pc[j,:, b,i*2], '.-',color = colors[b], label = baseRate)
                ax.set_ylabel('Percent pass')
                ax.set_xlabel('True RP')
                ax.set_title('contamination %.2f '%contRate)
        fig.text(0.65, 0.9-(.17*j), 'Recording Duration: %d hours'%recDur)
    fig.subplots_adjust(left=0.5, bottom=None, right=None, top=None, wspace=0.5, hspace=1.2)

    fig.legend(handles, labels, loc='upper right')
    fig.show()


# -*- coding: utf-8 -*-
"""
Created on Wed Aug  3 13:31:29 2022

@author: Noam Roth
scratch code to figure out whether amplitudes/frs matter for low fits
"""


from one.api import ONE
import ssl
ssl._create_default_https_context = ssl._create_unverified_context

savefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\savedSteinmetzFits_new\\'

one = ONE(cache_dir='E:\Steinmetz2019\Steinmetz_et_al_2019_9974357\9974357') # The location of the unarchived data
sessions = one.search()#dataset='trials') # search for all sessions that have a `trials` object
# session = sessions[0] # take the first session
# trials = one.load_object(session, 'trials')

#find sessions that have at least one channel in Cortex, Thalamus
# BRAIN_REGIONS = ['FRP','MO','SS','AUD','VIS','ACA','PL','ILA','ORB','RSP','TEa',
#                'CA1','CA2','CA3','DG',
#                'MB',
#                'DORsm','DORpm'] #change from Isocortext to 'CTX'??

BRAIN_REGIONS = ['ACA', 'AUD','ILA' , 'MOp', 'MOs',  'OLF', 'ORB', 'ORBm',
                     'PIR', 'PL', 'RSP', 'SSp','SSs',  'VISa', 'VISam', 'VISl',
                     'VISp', 'VISpm', 'VISrl',
                 'CA', 'CA1', 'CA2', 'CA3','DG', 'POST', 'SUB'
                 'TH', 'CL', 'LD', 'LGd', 'LH', 'LP', 'MD', 'MG','PO', 'POL', 
                     'PT','RT','SPF','VAL', 'VPL', 'VPM', 
                 'SNr','APN', 'IC','MB','MRN', 'NB','PAG','RN','SCig', 'SCm', 
                     'SCs', 'SCsg']
rpBinSize = 1 / 30000  
rpEdges = np.arange(0, 10/1000, rpBinSize) # in s  
rp = rpEdges + np.mean(np.diff(rpEdges)[0])/2 # vector of refractory period durations to test 
params = {}

for e,eid in enumerate(sessions[0:1]):
    
        print(f'processing {e + 1}/{len(sessions)}')
        
        print('Loading spike times and clusters...')
        # Load data
        try:
            spikes, clusters, channels =pickle.load(open("E:\Steinmetz2019\loaded_data\data_{eid}.p".format(eid=eid), "rb"))
        except FileNotFoundError:
            channels = one.load_object(eid,'channels')
            spikes = one.load_object(eid,'spikes')
            clusters = one.load_object(eid, 'clusters')
            pickle.dump((spikes, clusters, channels),(open("E:\Steinmetz2019\loaded_data\data_{eid}.p".format(eid=eid), "wb")))    
            

        #find all descendant acronyms of brain area of interest
        cluster_areas =[channels.brainLocation.allen_ontology[int(clusters.peakChannel[i])-1] for i in range(len(clusters.peakChannel))]
        cluster_idx = np.where(np.isin(cluster_areas, BRAIN_REGIONS))[0]
        brainRegions  = [cluster_areas[i] for i in cluster_idx]

        cluInds = np.array(list(x for x in range(len(spikes.clusters)) if spikes.clusters[x] in cluster_idx)) #this takes forever??
        spikeTimes = spikes.times[cluInds]
        spikeClusters = spikes.clusters[cluInds]
        
        
        print('Fitting RP Estimate...')
        # compute estimated RP for all neurons
        rpEstimates = fitSigmoidACG_All(spikeTimes, spikeClusters,brainRegions, rp, params)

        #add amp and fr info
        rpEstimates = collectAmpFR(spikes,cluInds,rpEstimates)


        # with open(savefile + eid + '.pickle', 'wb') as handle:
        #     pickle.dump(rpEstimates, handle)
#%%
fig,axs = plt.subplots(2,1)
axs[0].scatter(rpEstimates['rpEstimate'],rpEstimates['fr'])
axs[1].scatter(rpEstimates['rpEstimate'],rpEstimates['med_amp'])

#%%
def collectAmpFR(spikes,cluInds,rpEstimate):
    spikeTimes = spikes.times[cluInds]
    spikeClusters = spikes.clusters[cluInds]
    spikeAmps = spikes.amps[cluInds]
    
    
    cids = np.unique(spikeClusters)
    
    #add to dict
    rpEstimate['med_amp'] = []
    rpEstimate['fr']  = []

    for cidx in range(len(cids)):
        if cids[cidx] in rpEstimate['cidx']:
            st = spikeTimes[spikeClusters==cids[cidx]] 
            fr = len(st)/(st[-1]-st[0])
            amp = np.median(spikeAmps[spikeClusters==cids[cidx]] )
            rpEstimate['med_amp'].append(amp)
            rpEstimate['fr'].append(fr)

    
    return rpEstimate
    


# -*- coding: utf-8 -*-
"""
Created on Wed Aug 3 16:29:20 2022

@author: Noam Roth

example script to compute sliding refractory metric for one IBL session 
"""

import numpy as np
import matplotlib.pyplot as plt
from reproducible_ephys_functions import BRAIN_REGIONS, get_insertions, combine_regions
import ssl
ssl._create_default_https_context = ssl._create_unverified_context

from one.api import ONE
from slidingRP import *
import pickle
one = ONE()

from brainbox.io.one import SpikeSortingLoader
from ibllib.atlas import AllenAtlas
ba = AllenAtlas()

insertionsAll = get_insertions(level=2, one=one, freeze='biorxiv_2022_05')
saveLocally = False
# savefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\savedRSmetrics\\'

#set up params for slidingRP_all
params = {}
params['sampleRate'] = []
params['sampleRate'] = 30000
params['binSizeCorr'] = 1 / params['sampleRate']
params['returnMatrix'] = True
params['verbose'] = True
#%%
insertions = insertionsAll[0:1] #for this example, just run for a sample session 
all_recordings = []
for iIns, ins in enumerate(insertions):

    print(f'processing {iIns + 1}/{len(insertions)}')
    eid = ins['session']['id']
    probe = ins['probe_name']
    pid = ins['probe_insertion']

    # Load in spikesorting
    sl = SpikeSortingLoader(eid=eid, pname=probe, one=one, atlas=ba)
    spikes, clusters, channels = sl.load_spike_sorting()
    clusters = sl.merge_clusters(spikes, clusters, channels)

    #Find and load clusters from this insertion (in RS regions)
    clusters['rep_site_acronym'] = combine_regions(clusters['acronym'])
    # Clusters that are in the repeated site brain regions
    cluster_idx = np.sort(np.where(np.isin(clusters['rep_site_acronym'], 
                                           BRAIN_REGIONS))[0])

    print('Loading spike times and clusters...')
    cluInds = np.array(list(x for x in range(len(spikes.clusters))
                            if spikes.clusters[x] in cluster_idx)) 
    spikeTimes = spikes.times[cluInds]
    spikeClusters = spikes.clusters[cluInds]
    
    print('Running RP metric...')
    rpMetrics = slidingRP_all(spikeTimes, spikeClusters, params)
    
    
    if saveLocally:
        with open(savefile + ins['session']['subject'] + '.pickle', 'wb') as handle:
            pickle.dump(rpMetrics, handle)
        


# -*- coding: utf-8 -*-
"""
Created on Wed Aug  3 12:02:56 2022

@author: Noam Roth

Code to plot histograms of estimated RPs, as computed in computeEstimatedRPs.py

Runs for 3 datasets: IBL repeated site; Steinmetz 2019; Allen 

"""
#%% 

#set filter parameters for firing rate and amplitude across all 3 datasets
minFR = 20; minAmp = 100




#%%
#plot IBL data
one = ONE()
insertions = get_insertions(level=2, one=one, freeze='biorxiv_2022_05')

savefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\savedRSFits\\'

nSess = len(insertions)
plotEach = False
ca1All = []
dgAll = []
poAll = []
lpAll = []
visaAll = []



for s in range(nSess):
    subject = insertions[s]['session']['subject']
    #load saved rpMetrics
    try:
        file = open(savefile + subject + '.pickle','rb')
    except:
        continue
    rpEstimates = pickle.load(file)
    file.close()

    ca1 = np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['brainRegion'][i] == 'CA1' and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    po  = np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['brainRegion'][i] == 'PO' and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    dg  = np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['brainRegion'][i] == 'DG' and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    visa  = np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['brainRegion'][i] == 'PPC' and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    lp= np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['brainRegion'][i] == 'LP' and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    
    ca1[ca1<.05] =np.nan
    dg[dg<.05] = np.nan
    po[po<.05] =np.nan
    lp[lp<.05] = np.nan
    visa[visa<.05] =np.nan
    
    if plotEach:
        fig,axs = plt.subplots(2,2,figsize = (12,10))
        ax = axs[0,0]
        ax.hist(np.concatenate((ca1, dg)), 100)
        ax.set_title('hippocampus median: %.2f'%np.nanmedian(np.concatenate((ca1, dg))))
        ax = axs[0,1]
        ax.hist(np.concatenate((po, lp)), 100)
        ax.set_title('thalamus median: %.2f'%np.nanmedian(np.concatenate((po, lp))))
        
        ax = axs[1,0]
        ax.hist(visa, 100)
        ax.set_title('cortex median: %.2f'%np.nanmedian(visa))
    
    if len(ca1All) == 0:
        ca1All = ca1
        dgAll = dg
        lpAll = lp
        poAll = po
        visaAll = visa
    else:    
        ca1All = np.concatenate((ca1All,ca1))
        dgAll = np.concatenate((dgAll,dg))
        lpAll = np.concatenate((lpAll,lp))
        poAll = np.concatenate((poAll,po))
        visaAll = np.concatenate((visaAll,visa))
        
        

cortexAllRS = visaAll
thalamusAllRS = np.concatenate((poAll, lpAll))
hippocampusAllRS = np.concatenate((ca1All, dgAll))

#%%    plot all for rs 


fig,axs = plt.subplots(2,2,figsize = (8,6))
ax = axs[0,0]
ax.hist(cortexAllRS, 100)
ax.set_title('cortex (median rp: %.2f)'%np.nanmedian(cortexAllRS))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

ax = axs[0,1]
ax.hist(thalamusAllRS, 100)
ax.set_title('thalamus (median rp: %.2f)'%np.nanmedian(thalamusAllRS))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

ax = axs[1,0]
ax.hist(hippocampusAllRS, 100)
ax.set_title('hippocampus (median rp: %.2f)'%np.nanmedian(hippocampusAllRS))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

axs[1, 1].set_axis_off()
plt.tight_layout()
plt.suptitle('IBL (repeated site)',y=1.1)

fig.show()



#%% plot steinmetz
savefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\savedSteinmetzFits\\'

one = ONE(cache_dir='E:\Steinmetz2019\Steinmetz_et_al_2019_9974357\9974357') # The location of the unarchived data
sessions = one.search()#dataset='trials') # search for all sessions that have a `trials` object


BRAIN_REGIONS = ['ACA', 'AUD','ILA' , 'MOp', 'MOs',  'OLF', 'ORB', 'ORBm',
                     'PIR', 'PL', 'RSP', 'SSp','SSs',  'VISa', 'VISam', 'VISl',
                     'VISp', 'VISpm', 'VISrl',
                 'CA', 'CA1', 'CA2', 'CA3','DG', 'POST', 'SUB'
                 'TH', 'CL', 'LD', 'LGd', 'LH', 'LP', 'MD', 'MG','PO', 'POL', 
                     'PT','RT','SPF','VAL', 'VPL', 'VPM', 
                 'SNr','APN', 'IC','MB','MRN', 'NB','PAG','RN','SCig', 'SCm', 
                     'SCs', 'SCsg']
rpBinSize = 1 / 30000  
rpEdges = np.arange(0, 10/1000, rpBinSize) # in s  
rp = rpEdges + np.mean(np.diff(rpEdges)[0])/2 # vector of refractory period durations to test 
params = {}
nSess = len(sessions)
plotEach = False
cortexAll = []
hippocampusAll = []
thalamusAll = []
midbrainAll = []

for e,eid in enumerate(sessions):

    #load saved rpMetrics
    try:
        file = open(savefile + eid + '.pickle','rb')
    except:
        continue
    rpEstimates = pickle.load(file)
    file.close()



    #find neurons in each of the target brain regions that pass the firing rate and amplitude criteria
    hippocampus = np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['brainRegion'][i] in ['CA', 'CA1', 'CA2', 'CA3','DG', 'POST', 'SUB'] and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    cortex  = np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['brainRegion'][i] in ['ACA', 'AUD','ILA' , 'MOp', 'MOs',  'OLF', 'ORB', 'ORBm',
                     'PIR', 'PL', 'RSP', 'SSp','SSs',  'VISa', 'VISam', 'VISl',
                     'VISp', 'VISpm', 'VISrl'] and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    thalamus= np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['brainRegion'][i] in ['TH', 'CL', 'LD', 'LGd', 'LH', 'LP', 'MD', 'MG','PO', 'POL', 
                     'PT','RT','SPF','VAL', 'VPL', 'VPM' ] and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    midbrain = np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['brainRegion'][i] in ['SNr','APN', 'IC','MB','MRN', 'NB','PAG','RN','SCig', 'SCm',  'SCs', 'SCsg'] and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    
    
    #get rid of very low values #todo -- change this once filtered by neurons code is done!!
    hippocampus[hippocampus<.05] =np.nan
    thalamus[thalamus<.05] = np.nan
    cortex[cortex<.05] =np.nan
    midbrain[midbrain<.05] =np.nan
    
    #if this flag is True, plot the histogram for each session individually (sanity check)
    if plotEach:
        fig,axs = plt.subplots(2,2,figsize = (12,10))
        ax = axs[0,0]
        ax.hist(np.concatenate((ca1, dg)), 100)
        ax.set_title('hippocampus median: %.2f'%np.nanmedian(np.concatenate((ca1, dg))))
        ax = axs[0,1]
        ax.hist(np.concatenate((po, lp)), 100)
        ax.set_title('thalamus median: %.2f'%np.nanmedian(np.concatenate((po, lp))))
        
        ax = axs[1,0]
        ax.hist(visa, 100)
        ax.set_title('cortex median: %.2f'%np.nanmedian(visa))
    
    
    #concatenate each session's values for each region to a vector of all values for that region
    if len(hippocampusAll) == 0:
        hippocampusAll = hippocampus
    else:    
        hippocampusAll = np.concatenate((hippocampusAll, hippocampus))
        
    if len(cortexAll) == 0:
        cortexAll = cortex
    else:
        cortexAll = np.concatenate((cortexAll, cortex))

    if len(midbrainAll) == 0:
        midbrainAll = midbrain
    else:
        midbrainAll = np.concatenate((midbrainAll, midbrain))
        
    if len(thalamusAll) == 0:
        thalamusAll = thalamus
    else:
        thalamusAll = np.concatenate((thalamusAll, thalamus))

        
cortexAllSteinmetz = cortexAll
thalamusAllSteinmetz = thalamusAll
hippocampusAllSteinmetz = hippocampusAll
midbrainAllSteinmetz = midbrainAll
#%%   
#plot estimated RPs for all sessions from the Steinmetz 2019 dataset     
fig,axs = plt.subplots(2,2,figsize = (8,6))
ax = axs[0,0]
ax.hist(cortexAllSteinmetz, 100)
ax.set_title('cortex (median rp: %.2f)'%np.nanmedian(cortexAllSteinmetz))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

ax = axs[1,0]
ax.hist(hippocampusAllSteinmetz, 100)
ax.set_title('hippocampus (median rp: %.2f)'%np.nanmedian(hippocampusAllSteinmetz))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

ax = axs[0,1]
ax.hist(thalamusAllSteinmetz, 100)
ax.set_title('thalamus (median rp: %.2f)'%np.nanmedian(thalamusAllSteinmetz))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)


ax = axs[1,1]
ax.hist(midbrainAllSteinmetz, 100)
ax.set_title('midbrain (median rp: %.2f)'%np.nanmedian(midbrainAllSteinmetz))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

plt.tight_layout()
plt.suptitle('Steinmetz 2019',y=1.1)

fig.show()

 #%% polot allen       
rpBinSize = 1 / 30000  
rpEdges = np.arange(0, 10/1000, rpBinSize) # in s  
rp = rpEdges + np.mean(np.diff(rpEdges)[0])/2 # vector of refractory period durations to test 
params = {}
nSess = len(sessions)
plotEach = False
cortexAll = []
hippocampusAll = []
thalamusAll = []
midbrainAll = []

#find session names (from the saved files)
from os import listdir
from os.path import isfile, join
mypath = 'E:\AllenBrainObservatory\saved_units'
onlyfiles = [f for f in listdir(mypath) if isfile(join(mypath, f))]
sessions = [i[11:20] for i in onlyfiles]

for j, session_id in enumerate(sessions):
    file_name = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\savedAllenFits\%s.pickle'%session_id
    print('loading data for session %d out of %d'%(j+1, len(sessions)))

    #load the dataframe for this session
    with open(file_name, 'rb') as f:
        rpEstimates = pickle.load(f)


    hippocampus = np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['parentRegion'][i] == 'hippocampus' and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    cortex  = np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['parentRegion'][i] == 'cortex' and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    thalamus= np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['parentRegion'][i] == 'thalamus' and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    midbrain = np.array([rpEstimates['rpEstimate'][i] for i in range(len(rpEstimates['brainRegion'])) if rpEstimates['parentRegion'][i] == 'midbrain' and  rpEstimates['amp'][i]>minAmp and rpEstimates['fr'][i]>minFR])
    
    hippocampus[hippocampus<.05] =np.nan
    thalamus[thalamus<.05] = np.nan
    cortex[cortex<.05] =np.nan
    midbrain[midbrain<.05] =np.nan
    
    if plotEach:
        fig,axs = plt.subplots(2,2,figsize = (12,10))
        ax = axs[0,0]
        ax.hist(np.concatenate((ca1, dg)), 100)
        ax.set_title('hippocampus median: %.2f'%np.nanmedian(np.concatenate((ca1, dg))))
        ax = axs[0,1]
        ax.hist(np.concatenate((po, lp)), 100)
        ax.set_title('thalamus median: %.2f'%np.nanmedian(np.concatenate((po, lp))))
        
        ax = axs[1,0]
        ax.hist(visa, 100)
        ax.set_title('cortex median: %.2f'%np.nanmedian(visa))
    
    if len(hippocampusAll) == 0:
        hippocampusAll = hippocampus
    else:    
        hippocampusAll = np.concatenate((hippocampusAll, hippocampus))
        
    if len(cortexAll) == 0:
        cortexAll = cortex
    else:
        cortexAll = np.concatenate((cortexAll, cortex))

    if len(midbrainAll) == 0:
        midbrainAll = midbrain
    else:
        midbrainAll = np.concatenate((midbrainAll, midbrain))
        
    if len(thalamusAll) == 0:
        thalamusAll = thalamus
    else:
        thalamusAll = np.concatenate((thalamusAll, thalamus))

        
cortexAllAllen = cortexAll
thalamusAllAllen = thalamusAll
hippocampusAllAllen = hippocampusAll
midbrainAllAllen = midbrainAll



#%%        
fig,axs = plt.subplots(2,2,figsize = (8,6))
ax = axs[0,0]
ax.hist(cortexAllAllen, 100)
ax.set_title('cortex (median rp: %.2f)'%np.nanmedian(cortexAllAllen))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

ax = axs[0,1]
ax.hist(thalamusAllAllen, 100)
ax.set_title('thalamus (median rp: %.2f)'%np.nanmedian(thalamusAllAllen))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

ax = axs[1,0]
ax.hist(hippocampusAllAllen, 100)
ax.set_title('hippocampus (median rp: %.2f)'%np.nanmedian(hippocampusAllAllen))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)


ax = axs[1,1]
ax.hist(midbrainAllAllen, 100)
ax.set_title('midbrain (median rp: %.2f)'%np.nanmedian(midbrainAllAllen))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

plt.tight_layout()
plt.suptitle('Allen',y=1.1)

fig.show()


#%%

#concatenate all datasets

cortexAllDatasets = np.concatenate((cortexAllRS, cortexAllSteinmetz, cortexAllAllen))
midbrainAllDatasets = np.concatenate((midbrainAllSteinmetz, midbrainAllAllen))
thalamusAllDatasets = np.concatenate((thalamusAllRS, thalamusAllSteinmetz, thalamusAllAllen))
hippocampusAllDatasets = np.concatenate((hippocampusAllRS, hippocampusAllSteinmetz,hippocampusAllAllen))





fig,axs = plt.subplots(2,2,figsize = (8,6))
ax = axs[0,0]
ax.hist(cortexAllDatasets, 100)
ax.set_title('cortex (median rp: %.2f)'%np.nanmedian(cortexAllDatasets))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

ax = axs[0,1]
ax.hist(thalamusAllDatasets, 100)
ax.set_title('thalamus (median rp: %.2f)'%np.nanmedian(thalamusAllDatasets))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

ax = axs[1,0]
ax.hist(hippocampusAllDatasets, 100)
ax.set_title('hippocampus (median rp: %.2f)'%np.nanmedian(hippocampusAllDatasets))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)


ax = axs[1,1]
ax.hist(midbrainAllDatasets, 100)
ax.set_title('midbrain (median rp: %.2f)'%np.nanmedian(midbrainAllDatasets))
ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

plt.tight_layout()
plt.suptitle('All Datasets',y=1.1)
fig.show()



#%%

#non-smoothed histograms, all data


fig,axs = plt.subplots(1,1,figsize = (5,3))
ax = axs
ax.hist(cortexAllDatasets, 100, histtype = 'step', color = 'blue', label = 'Cortex')
ax.hist(thalamusAllDatasets, 100, histtype = 'step', color = 'green', label = 'Thalamus')
ax.hist(hippocampusAllDatasets, 100,  histtype = 'step', color = 'purple', label = 'Hippocampus')

ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Number of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)


plt.legend()
plt.tight_layout()
plt.suptitle('All Datasets',y=1.1)
fig.show()






#%%
from scipy.ndimage.filters import gaussian_filter1d

nBins = 20
fig,axs = plt.subplots(1,1,figsize = (5,3))
ax = axs

def plot_hists(ax, data, color, linestyle, label= None):
    cc = data[~np.isnan(data)]
    xx = sum(~np.isnan(data))
    h, b = np.histogram(cc, bins=nBins, weights = np.ones(xx) / xx, density=False)
    x = b[:-1]
    y = gaussian_filter1d(h, 1)
    ax.plot(x, y, color=color,linestyle = linestyle, label = label)

plot_hists(ax, cortexAllRS, 'blue', '-', 'cortex')
plot_hists(ax, thalamusAllRS, 'green', '-', 'thalamus')
plot_hists(ax, hippocampusAllRS, 'purple', '-', 'hippocampus')

plot_hists(ax, cortexAllSteinmetz, 'blue', '--')
plot_hists(ax, thalamusAllSteinmetz, 'green', '--')
plot_hists(ax, hippocampusAllSteinmetz, 'purple', '--')

plot_hists(ax, cortexAllAllen, 'blue', ':')
plot_hists(ax, thalamusAllAllen, 'green', ':')
plot_hists(ax, hippocampusAllAllen, 'purple', ':')

ax.set_xlabel('Estimated RP (ms)')
ax.set_ylabel('Proportion of neurons')
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)

def add_median_arrows(ax, data, y_value, lenArrow, lenHead, wiArrow, color, linestyle):
    ind = np.nanmedian(data)
    ax.annotate('', xy=(ind, y_value), xytext=(ind,y_value + lenArrow + lenHead),
        arrowprops={'arrowstyle': '->','color':color, 'ls': linestyle})
    
#add arrows:
lenArrow = .05
lenHead = 0
wiArrow = 0
ind = np.nanmedian(cortexAllRS)
n = 0.2#len(cortexAll[(cortexAll > ind*0.95) & (cortexAll <  ind*1.05)])/2 #y value of overall histogram

add_median_arrows(ax, cortexAllRS, n, lenArrow, lenHead, wiArrow, 'blue', 'solid')
add_median_arrows(ax, cortexAllSteinmetz, n, lenArrow, lenHead, wiArrow, 'blue', 'dashed')
add_median_arrows(ax, cortexAllAllen, n, lenArrow, lenHead, wiArrow, 'blue', 'dotted')

add_median_arrows(ax, thalamusAllRS, n,  lenArrow, lenHead, wiArrow, 'green', 'solid')
add_median_arrows(ax, thalamusAllSteinmetz, n, lenArrow, lenHead, wiArrow, 'green', 'dashed')
add_median_arrows(ax, thalamusAllAllen, n, lenArrow, lenHead, wiArrow, 'green', 'dotted')

add_median_arrows(ax, hippocampusAllRS, n, lenArrow, lenHead, wiArrow, 'purple', 'solid')
add_median_arrows(ax, hippocampusAllSteinmetz, n, lenArrow, lenHead, wiArrow, 'purple', 'dashed')
add_median_arrows(ax, hippocampusAllAllen, n, lenArrow, lenHead, wiArrow, 'purple', 'dotted')




ax.plot(np.NaN, np.NaN, '-', color='black', label='IBL')
ax.plot(np.NaN, np.NaN, '--', color='black', label='Steinmetz 2019')
ax.plot(np.NaN, np.NaN, ':', color='black', label='Allen')

plt.legend(frameon=False)
plt.tight_layout()
fig.show()

plt.savefig(r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\estimatedRPs%dAmp_%dFR.pdf'%(minAmp,minFR), dpi=300, format='pdf')


#%% TODO tomorrow:
    
    # scatter plots: rpEstimate vs fr and rpEstimate vs amplitude
    # histograms with different subselections: change amp and fr and run this code for a few different seleections of the parameters
    # if time: look into low thalamus example neurons!


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    require = [x.strip() for x in f.readlines() if not x.startswith('git+')]

setuptools.setup(
    name="slidingRP",
    version="1.0.0",
    author="Noam Roth",
    description="Quality metric from spike trains",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/oliche/viewephys",
    project_urls={
        "Bug Tracker": "https://github.com/SteinmetzLab/slidingRefractory/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=require,
    package_dir={'': 'python'},
    packages=setuptools.find_packages(where="python"),
#    package_data={'viewephys': ['raster.ui', 'nav_file.ui', 'viewephys.svg']},
    python_requires=">=3.8",
)



# -*- coding: utf-8 -*-
"""
Created on Thu Jul 21 08:29:37 2022

@author: Noam Roth

script for running and plotting various simulations
"""

import simulations
from simulations import *
import pickle
import numpy as np
from phylib.stats import correlograms
from types import SimpleNamespace
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import numpy as np
from scipy import stats
from scipy.optimize import curve_fit
import time
from slidingRP import *


#%%
#script testing (to be moved later for import purposes)

# from simulations import genST  # this is not working, for some reason it's importing a different simulations gensT
baseRate = 10 #spks/s
recDur = 3600 #seconds
params = {}
params['checkFR']  = 1
st = genST(meanRate, recDur, xx = None, params = params)

isi = np.diff(np.insert(st,0,0)) #add a zero to the beginning to includ the "first" isi? 
isi = np.delete(isi,np.where(isi<rp)[0]) 
st = np.cumsum(isi)
if c>0:
    contST = genST(contRate,recDur)
else:
    contST=[]
combST = np.sort(np.concatenate((st, contST)))



#%%
sampleRate = 30000
params = {
    'recDurs': np.array([0.25, 0.5, 1, 2, 5]),  #recording durations (hours)
    'RPs': np.array([0.001,0.0015, 0.002, 0.0025, 0.003, 0.004, 0.005]), #true RP (s)
    'baseRates': np.array([0.1, 0.25, 0.5, 1, 5, 10,20]), #F1, 2, 5, 10 , 20 R (spk/s)
    'contRates':  np.arange(0.00,0.225, 0.01), #contamination levels (proportion) #.025
    'nSim': 20,
    'threshold': 0.1,
    'binSize': 1 / sampleRate,
    'sampleRate': 30000,  #TODO figure out a way to refer to this in binsize?
    'checkFR': False,
    'binSizeCorr': 1 / sampleRate,
    'returnMatrix': True,
    'verbose': True ,
    'savePCfile': True
    
}
#%%
[pc, pc2MsNoSpikes] = simulateContNeurons(params)
import datetime

date_now  = datetime.datetime.now().strftime('_%m_%d')

savefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\simulationsPC' + str(params['nSim']) + 'iter' + date_now + '.pickle'


results = [pc, pc2MsNoSpikes, params]
if params['savePCfile']:
    with open(savefile, 'wb') as handle:
        pickle.dump(pc, handle)
     
        
     
        #%%
savefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\simulationsPC' + str(params['nSim']) + 'iter' + date_now + '.pickle'
#load data
file = open(savefile,'rb')
results = pickle.load(file)
file.close()

#prep for figure saving
figsavefile1 = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\simulationsPC' + str(params['nSim']) + 'iter' + date_now + '.pdf'
figsavefile2 = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\simulationsPC' + str(params['nSim']) + 'iter' + date_now + '2MsNoSpikes.pdf'


plotSimulations(pc, params, figsavefile1)
plotSimulations(pc2MsNoSpikes, params, figsavefile2)
#%%
savefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\simulationsPC20iter.pickle'

sampleRate = 30000
params = {
    'recDurs': np.array([0.25, 0.5, 1, 2, 5]),  #recording durations (hours)
    'RPs': np.array([0.001,0.0015, 0.002, 0.003, 0.004, 0.005]), #true RP (s)
    'baseRates': np.array([ 0.5, 1, 2, 5, 10, 20 ]), #F1, 2, 5, 10 , 20 R (spk/s)
    'contRates':  np.arange(0.00,0.225, 0.025), #contamination levels (proportion) #.025
    'nSim': 20,
    'threshold': 0.1,
    'binSize': 1 / sampleRate,
    'sampleRate': 30000,  #TODO figure out a way to refer to this in binsize?
    'checkFR': False,
    'binSizeCorr': 1 / sampleRate,
    'returnMatrix': True,
    'verbose': True ,
    'savePCfile': True
    
}


file = open(savefile,'rb')
pc = pickle.load(file)
file.close()

figsavefile = r'C:\Users\Steinmetz Lab User\Documents\GitHub\analysis\slidingRefractory\python\simulationsPC20iter.pdf'
plotSimulations(pc, params, figsavefile)



#%%
#test a closed form expression for probability of passing
P = 0.1 #true contamination
R = 1
r = 0.002 #true frefractory period
D = 2*3600; #recording duration


B = S*R*P*r
S = R*D

lam = P* R * D * S * S 
C = 1-stats.poisscdf(B, lam)

#%%

#define params
contRate = 0.1
rp = 0.002
baseRate = 4

recDur = 2*3600
nSim = 1000


rpBinSize = 1 / 30000  
rpEdges = np.arange(0, 10/1000, rpBinSize) # in s  
rpVec = rpEdges + np.mean(np.diff(rpEdges)[0])/2 #

obsViol = np.empty(nSim)
obsViol[:] = np.nan
expectedViol = np.empty(nSim)
expectedViol[:] = np.nan
for i in range(nSim):
    #generate contaminated spiketrain 
    st = genST(baseRate,recDur,params) #generate a spike train with the current base rate
    isi = np.diff(np.insert(st,0,0)) 
    isi = np.delete(isi,np.where(isi<rp)[0]) #get rid of already contaminating spikes (does this make sense??? why are there already contaminating spikes)
    st = np.cumsum(isi)
    if contRate>0: #contamination greater than 0 
        contST = genST(contRate*baseRate,recDur,params)
    else:
        contST=[]
    combST = np.sort(np.concatenate((st, contST))) # put spike times from both spike trains together (and sort chronologically)
    
    st = combST
    spikeCount = len(st)
    
    
    #compute firing rate 
    clustersIds = [0] # call the cluster id 0 (not used, but required input for correlograms)
    spikeClustersACG = np.zeros(len(st), dtype = 'int8') # each spike time gets cluster id 0 
    nACGFR = correlograms(st, spikeClustersACG, cluster_ids = clustersIds, bin_size = 1, sample_rate = 30000, window_size=2,symmetrize=False)[0][0] #compute acg
    firingRate = nACGFR[1] / spikeCount
      
    #compute ACG
    nACG = correlograms(st, spikeClustersACG, cluster_ids = clustersIds, bin_size = params['binSizeCorr'], sample_rate = params['sampleRate'], window_size=2,symmetrize=False)[0][0] #compute acg
  
    # compute observed violations  
    rpIdx = np.where(rpVec > rp)[0][0]
    obsViol[i] = sum(nACG[0:rpIdx])
    # print('obsViol: ', obsViol)
    
    
    expectedViol[i] = firingRate * contRate * rp * 2 * spikeCount 
    # print('expectedViol: ', expectedViol)
    
#%%

#%% test the 'expected count'

baseRates = np.logspace(-0.6,1.5,80)
recDur = 7200; 
rp = 0.002;
contProp = 0.1;
# params = struct(); params.cont = 10;
obsViol = np.empty(len(baseRates))
obsViol[:] = np.nan
expViol = np.empty(len(baseRates))
expViol[:] = np.nan
for bidx in range(len(baseRates)):
    baseRate = baseRates[bidx];
    contRate = baseRate*contProp;
    
    st = genST(baseRate,recDur,params);  
    isi = np.diff(np.insert(st,0,0)) 
    isi = np.delete(isi,np.where(isi<rp)[0]) #get rid of already contaminating spikes (does this make sense??? why are there already contaminating spikes)
    st = np.cumsum(isi)
    
    
    
    contST = genST(contRate,recDur,params)
    
    combST = np.sort(np.concatenate((st, contST))) # put spike times from both spike trains together (and sort chronologically)
    
    [ confMatrix, cont, rpTestVals, nACG, firingRate]= computeMatrix(combST, params)
    
    nACG = nACG[0:len(rpTestVals)]
    contaminationRate = firingRate*contProp; 
    expectedViol = contaminationRate*rp*2*len(combST);
    
    obsViol[bidx] = sum(nACG[rpTestVals<=0.002]); 
    expViol[bidx] = expectedViol;

#%%
fig,ax= plt.subplots(1,1)
ax.plot(baseRates, obsViol, 'r.-',label = 'observed violations'); 
ax.plot(baseRates, expViol, 'k.-',label = 'expected violations');
ax.set_xlabel('Base firing rate (spks/s)')
ax.set_ylabel('Number of violations'); 
ax.spines.right.set_visible(False)
ax.spines.top.set_visible(False)
fig.legend()
#%%



import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from scipy.special import factorial
from scipy.stats import poisson

bins = np.arange(25) - 0.5
entries, bin_edges, patches = plt.hist(obsViol, bins=bins, density=True, label='Data')

# calculate bin centres
bin_middles = 0.5 * (bin_edges[1:] + bin_edges[:-1])


def fit_function(k, lamb):
    '''poisson function, parameter lamb is the fit parameter'''
    return poisson.pmf(k, lamb)


# fit with curve_fit
parameters, cov_matrix = curve_fit(fit_function, bin_middles, entries)
x_plot = np.arange(0, 25)

plt.plot(
    x_plot,
    fit_function(x_plot, *parameters),
    marker='o', linestyle='',
    label='Fit result',
)
plt.legend()
plt.title('Observed Viols (mean: %.2f) over %d simulations. Mean of ExpViol: %.2f'%(parameters[0],nSim, np.mean(expectedViol)))

plt.show()


#%%
plt.plot([0.5,1,2,3,4],[1.66,3.18,6.21,9.88,13.57],'r.-',label = 'observed violations')
plt.plot([0.5,1,2,3,4],[1.03, 3.47,12.61,27.37,47.66], 'k.-', label = 'expected violations')
plt.xlabel('Firing Rate')
plt.ylabel('Number of spikes (averaged across simulations)')
plt.title('Number of violations: contRate = 0.1; rp = 0.002; recDur = 2hr')
plt.legend()


#%%

fig,axs = plt.subplots(1,3)
axs[0].hist(obsViol)
axs[0].set_title('obsViol')
axs[1].hist(expectedViol)
axs[1].set_title('expectedViol')

conf = 1 - stats.poisson.cdf(obsViol, expectedViol)
axs[2].hist(conf)
axs[2].set_title('Confidence')




percentPass = sum(conf>confThresh)/len(conf)


#now test Nick's code
PLimit = 0.1
PTrue = contRate
R = firingRate
r = rp
S = spikeCount
confLimit = 0.1

S = firingRate*recDur#R*D; % spike count
violLimit = PLimit*R*r*2*S; # mean count of a poisson process at the limit of acceptable contamination
violTrue = PTrue*R*r*2*S; # mean count of poisson process at the true contamination --> this one is not right


# % what is the value of the limit contamination's CDF that would exceed the
# % acceptable confidence? I.e. when does the limit CDF = 10%? this is the
# % max acceptable.
k = stats.poisson.ppf(1-confLimit, violLimit);

# % now what is the value of the true CDF just below that value? that's the
# % proportion of simulations that should pass
propPass = stats.poisson.cdf(k-1, violTrue); % k can be negative, in which case it returns zero